#include <time.h>
#include "CheckFrame.hh"
#include <stdio.h>
using namespace std;

CheckFrame::CheckFrame(istream& in)
  : mFile(in), mDebug(0)
{
    //---------------------------------  Initialize the instance counts.
    for (int i=0 ; i<MaxID ; i++) {
        mInstCt[i] = 0;
	mDefID[i]  = false;
	mProcID[i] = false;
    }
    mDefID[1] = true;
    mDefID[2] = true;
}

CheckFrame::~CheckFrame() {
}

void
CheckFrame::Check(void) throw(BadFile) {
    char line[128];

    //---------------------------------  Check the IGWD specifier
    mFile.ReadHeader();
    int hdrSz = mFile.getStrucHdrLen();

    //---------------------------------  Loop over structures
    while (1) {

        //-----------------------------  Read the structure header
        mFile.NxStruct();

        //-----------------------------  Check for valid structure length
	if (mFile.getLength() < hdrSz) {
	    sprintf(line, "Illegal structure length %i", mFile.getLength());
	    throw BadFile(line);
	}

        //-----------------------------  Check for valid structure ID
	int id = mFile.getID();
	if (id <= 0 || id >= MaxID || !mDefID[id]) {
	    sprintf(line, "Illegal or undefined structure ID %i", id);
	    throw BadFile(line);
	}

        //-----------------------------  Check for valid instance number
	int inst = mFile.getInstance();
	if (inst != mInstCt[id]++) {
	    if (inst == 1 && mInstCt[id] == 1) {
	        sprintf(line, 
			"Warning: First instance of structure %i is 1", 
			id);
		ErrorOut(line);
	        mInstCt[id]++;
	    } else {
		sprintf(line, 
			"Illegal structure %i instance %i (%i expected)", 
			id, inst, mInstCt[id]-1);
		throw BadFile(line);
	    }
	}

	if (mDebug>1) {
	    sprintf(line, 
		    "Length, id, instance = %i %i %i", 
		    mFile.getLength(), id, inst);
	    ErrorOut(line);
	}

	if (id == 1) {
            procSH();
	} else if (mProcID[id]) {
	    procID();
	}

	if (mFile.getID() == mEOFrID) {
	    if (mFile.getVersion() >= 4 && mFile.getVersion() < 6) {
	        for (int i=1 ; i<MaxID ; i++) mInstCt[i] = 0;
	    }
	}
	if (mFile.getID() == mEOFID) break;
    }
    if (mDebug) ErrorOut("End of Frame.");
}

void
CheckFrame::procSH(void) {
    char line[128];
    string name(mFile.getString());

    unsigned short ID = mFile.getShort();

    string comment(mFile.getString());
    if (mProcID[mFile.getID()]) {
        sprintf(line, "Structure type SH, Instance %i, Length %i",
		mFile.getInstance(), mFile.getLength());
	ErrorOut(line);
	sprintf(line, "  Name:%-20s  ID: %4i Comment: %s", name.c_str(), 
		ID, comment.c_str());
	ErrorOut(line);
    }

    if (!ID || ID > MaxID) throw BadFile("Illegal structure ID defined");

    mDefID[ID] = true;
    if (name == "FrEndOfFile")  mEOFID = ID;
    if (name == "FrEndOfFrame") mEOFrID = ID;
}

void
CheckFrame::procID(void) {
}

void
CheckFrame::ErrorOut(const string& error) {
    cout << error << endl;
}

