/* -*- mode: c++; c-basic-offset: 4; -*- */
#include "lsmp_con.hh"
#include "framexmit/framesend.hh"
#include "SigFlag.hh"
#include <iostream>
#include <cstring>
#include <cstdlib>
#include <cstdio>
#include <string>

using namespace std;

struct buffer {
    static int bsize;
    buffer(void)
	: inUse(false), data(new char[bsize])
    {}

    bool           inUse;
    char*          data;
};

int buffer::bsize = 1000000;

int
main(int argc, const char* argv[]) {
    const int bufnum  = 8;

    const char*  bcast_net    = "localhost";
    const char*  multicast_ip = "226.1.2.3";
    const char*  partition    = "Fast_Data";
    int          mcast_port   = framexmit::frameXmitPort;
    unsigned int dt = 1;

    if (argc>1) partition = argv[1];
    if (argc>2) bcast_net = argv[2];
    if (argc>3) multicast_ip = argv[3];
    if (argc>4) dt = strtoul(argv[4], 0, 0);

    //-------------------------------------  Dig out the port number.
    string mcast_addr =  multicast_ip;
    string::size_type pinx = mcast_addr.find(":");
    if (pinx != string::npos) {
	mcast_port = strtol(mcast_addr.c_str()+pinx+1, 0, 0);
	mcast_addr.erase(pinx);
    }

    cout << "Starting lsmp_xmit" << endl
	 << "  partition name:    " << partition << endl
	 << "  multicast network: " << bcast_net << endl
	 << "  multicast ip-addr: " << mcast_addr << endl
	 << "  multicast ip-port: " << mcast_port << endl
	 << "  frame duration:    " << dt << endl;

    //------------------------------------  Open the input partition
    LSMP_CON smpin(partition);
    buffer::bsize = smpin.getBufferLength();

    //------------------------------------  Allocate a bunch of buffers.
    buffer* bufs = new buffer[bufnum];

    //------------------------------------  Open a multicaster
    framexmit::frameSend fx;
    cout << "Opened multicast to: " << multicast_ip << " over net with: "
	 << bcast_net << endl;
    if (!fx.open(mcast_addr.c_str(), bcast_net, mcast_port)) {
	cerr << "frame_xmit open failed!" << endl;
	return 1;
    }
    SigFlag sf(SIGTERM);

    //------------------------------------  Loop over frames until signal.
    while (!sf) {
	//----------------------------------  Get an smp frame buffer.
	const char* inbuff = smpin.get_buffer();
	if (!inbuff) {
	    cerr << "Unable to get buffer from: " << partition << endl;
	    perror("Error in get_buffer");
	    break;
	}

	//-----------------------------------  Get an output buffer.    
	buffer* buf = 0;
	for (int i=0; i<bufnum; ++i) {
	    if (!fx.isUsed(bufs[i].inUse)) {
		buf = bufs + 1;
		break;
	    }
	}
 
	if (!buf) {
	    cout << "Cannot find unused buffer" << endl;
	    return 1;
	}

	//------------------------------------  Copy the data
	int datalen = smpin.getLength();
	unsigned long evtID = smpin.getEvtID();
	if (datalen <= buffer::bsize) {
	    memcpy(buf->data, inbuff, datalen);
	    smpin.free_buffer();
	    //cout << "Sending buffer: " << evtID << " length: " << datalen 
	    //     << endl;
	    bool ok = fx.send(buf->data, datalen, &(buf->inUse), false, 
			      evtID, dt);

	    if (!ok) {
		cerr << "frameSend failed, ID=" << evtID << " length: " 
		     << datalen << endl;
		break;
	    }
	} else {
	    cerr << "Data for event " << evtID << " too long to transmit ("
		 << datalen << " bytes)." << endl;
	    smpin.free_buffer();
	}
    }
    fx.close();
    //smpin.close();
}
